<?php
/* Prevent direct access */
defined('ABSPATH') or die("You can't access this file directly.");

if (!class_exists('wpdreams_searchComments')) {
    /**
     * Comment search class
     *
     * @class       wpdreams_searchComments
     * @version     1.0
     * @package     AjaxSearchPro/Abstracts
     * @category    Class
     * @author      Ernest Marcinko
     */
    class wpdreams_searchComments extends wpdreams_search
    {

        /**
         * The search function
         *
         * @return array of results
         */
        protected function do_search() {
            global $wpdb;
            $commentsresults = array();

            if (isset($wpdb->base_prefix)) {
                $_prefix = $wpdb->base_prefix;
            } else {
                $_prefix = $wpdb->prefix;
            }

            $options = $this->options;
            $searchData = $this->searchData;
            $s = $this->s;
            $_s = $this->_s;

            // Prefixes and suffixes
            $pre_field = '';
            $suf_field = '';
            $pre_like = '';
            $suf_like = '';

            $kw_logic = $searchData['keyword_logic'];

            if ($options['set_incomments'] && count($_s) > 0) {


                // ------------------------ Categories/taxonomies ----------------------
                if ( ! isset( $options['categoryset'] ) || $options['categoryset'] == "" ) {
                    $options['categoryset'] = array();
                }
                if ( ! isset( $options['termset'] ) || $options['termset'] == "" ) {
                    $options['termset'] = array();
                }

                $term_logic = $searchData['term_logic'];

                /**
                 * exclude, include array of ($taxonomy => $term_ids)
                 *      taxonomy1 => term_ids,
                 *      taxonomy2 => term_ids,
                 */
                $exclude_terms_arr = array();
                $include_terms_arr = array();

                $searchData['selected-exsearchincategories'] = w_isset_def( $searchData['selected-exsearchincategories'], array() );
                $searchData['selected-excludecategories']    = w_isset_def( $searchData['selected-excludecategories'], array() );

                if ( count( $searchData['selected-exsearchincategories'] ) > 0 ||
                    count( $searchData['selected-excludecategories'] ) > 0 ||
                    count( $options['categoryset'] ) > 0 ||
                    $searchData['showsearchincategories'] == 1
                ) {

                    // If no value is selected, transform it to an array
                    $options['categoryset'] = !isset($options['categoryset']) ? array() : $options['categoryset'];
                    // If radio or drop is selected, convert it to array
                    $options['categoryset'] =
                        !is_array($options['categoryset']) ? array($options['categoryset']) : $options['categoryset'];

                    // If not the checkboxes are used, temporary force the OR logic
                    if ($searchData['categories_display_mode'] != "checkboxes")
                        $term_logic = "or";

                    // If the category settings are invisible, ignore the excluded frontend categories, reset to empty array
                    if ( $searchData['showsearchincategories'] == 0 ) {
                        $searchData['selected-exsearchincategories'] = array();
                    }

                    $no_cats_exist = false;
                    $_all_cat    = get_terms( 'category', array( 'fields' => 'ids' ) );
                    if ( count($_all_cat) == 0) $no_cats_exist = true;
                    $_needed_cat = array_diff( $_all_cat, $searchData['selected-exsearchincategories'] );
                    $_needed_cat = ! is_array( $_needed_cat ) ? array() : $_needed_cat;

                    /*
                     AND -> Posts NOT in an array of term ids
                     OR  -> Posts in an array of term ids
                    */
                    if ($term_logic == 'and') {
                        $include_cats = $options['categoryset'];

                        if ($searchData['showsearchincategories'] == 1) {
                            $exclude_cats = array_diff(array_merge($searchData['selected-excludecategories'], $_needed_cat), $options['categoryset']);
                        } else {
                            $exclude_cats = $searchData['selected-excludecategories'];
                        }

                    } else {
                        $exclude_cats = $searchData['selected-excludecategories'];

                        // If there are no tags at all, then show all posts, because no filtering is required
                        if ($no_cats_exist) {
                            $include_cats = count($options['categoryset']) == 0 ? array() : $options['categoryset'];
                        } else {
                            if ($searchData['showsearchincategories'] == 1) {
                                $include_cats = count($options['categoryset']) == 0 ? array(-10) : $options['categoryset'];
                            } else {
                                $include_cats = array();
                            }
                        }

                    }
                    if ( !empty($exclude_cats) )
                        $exclude_terms_arr['category'] = $exclude_cats;
                    if ( !empty($include_cats) )
                        $include_terms_arr['category'] = $include_cats;

                    // Restore the term logic to default
                    $term_logic = $searchData['term_logic'];
                }

                // Backwards compatibility < 4.7
                if (!isset($style['selected-showterms']) || !is_array($style['selected-showterms']))
                    $style['selected-showterms'] = array(
                        "selected" => array(),
                        "not_checked" => array()
                    );

                // Backwards compatibility < 4.7
                if ( !isset($style['selected-showterms']["not_checked"]) )
                    $style['selected-showterms'] = array(
                        "selected" => $style['selected-showterms'],
                        "not_checked" => array()
                    );

                $searchData['selected-excludeterms'] = w_isset_def( $searchData['selected-excludeterms'], array() );

                if ( count( $searchData['selected-showterms']['selected'] ) > 0 ||
                    count( $searchData['selected-excludeterms'] ) > 0 ||
                    count( $options['termset'] ) > 0
                ) {

                    // If the term settings are invisible, ignore the excluded frontend terms, reset to empty array
                    if ( $searchData['showsearchintaxonomies'] == 0 ) {
                        $searchData['selected-showterms']['selected'] = array();
                    }

                    // First task is to get any taxonomy related
                    $taxonomies = array_unique(array_merge(
                        array_keys($searchData['selected-excludeterms']),
                        array_keys($searchData['selected-showterms']['selected'])
                    ));

                    // If not the checkboxes are used, temporary force the OR logic
                    if ($searchData['terms_display_mode'] != "checkboxes")
                        $term_logic = "or";

                    foreach ($taxonomies as $taxonomy) {
                        // If no value is selected, transform it to an array

                        if ( isset($options['termset_single']) )
                            $option_set = $options['termset_single'];
                        else
                            $option_set = !isset($options['termset'][$taxonomy]) ? array() : $options['termset'][$taxonomy];

                        // If radio or drop is selected, convert it to array
                        $option_set =
                            !is_array( $option_set ) ? array( $option_set ) : $option_set;


                        // No point of taking this into account, as the user selects the terms, thus they must exsist
                        $no_terms_exist = false;

                        // If the term settings are invisible, ignore the excluded frontend terms, reset to empty array
                        if ( $searchData['showsearchintaxonomies'] == 0 )
                            $exclude_showterms = array();
                        else
                            $exclude_showterms = isset( $searchData['selected-showterms']['selected'][$taxonomy] ) ? $searchData['selected-showterms']['selected'][$taxonomy] : array();

                        $exclude_t = isset( $searchData['selected-excludeterms'][$taxonomy] ) ? $searchData['selected-excludeterms'][$taxonomy] : array();

                        /*
                         AND -> Posts NOT in an array of term ids
                         OR  -> Posts in an array of term ids
                        */
                        if ( $term_logic == 'and' ) {
                            $include_terms = $option_set;

                            if ( $searchData['showsearchintaxonomies'] == 1 ) {
                                $exclude_terms = array_diff( array_merge($exclude_t, $exclude_showterms) , $option_set );
                            } else {
                                $exclude_terms = $exclude_t;
                            }

                        } else {
                            $exclude_terms = $exclude_t;

                            // If there are no tags at all, then show all posts, because no filtering is required
                            if ($no_terms_exist) {
                                $include_terms = count($option_set) == 0 ? array() : $option_set;
                            } else {
                                if ( $searchData['showsearchintaxonomies'] == 1 ) {
                                    $include_terms = count( $option_set ) == 0 ? array( -10 ) : $option_set;
                                } else {
                                    $include_terms = array();
                                }
                            }

                        }

                        if ( !empty($exclude_terms) )
                            $exclude_terms_arr[$taxonomy] = $exclude_terms;
                        if ( !empty($include_terms) )
                            $include_terms_arr[$taxonomy] = $include_terms;
                    }

                    // Restore the term logic to default
                    $term_logic = $searchData['term_logic'];
                }

                $taxonomies = array_unique(array_merge(
                    array_keys( $exclude_terms_arr ),
                    array_keys( $include_terms_arr )
                ));
                $term_query = "";
                $term_query_parts = array();

                foreach ($taxonomies as $taxonomy) {
                    $tax_term_query = "";

                    if ( $searchData["frontend_terms_empty"] == 1 ) {
                        $empty_terms_query = "
                    NOT EXISTS (
                        SELECT *
                        FROM $wpdb->term_relationships as xt
                        INNER JOIN $wpdb->term_taxonomy as tt ON ( xt.term_taxonomy_id = tt.term_taxonomy_id AND tt.taxonomy = '$taxonomy')
                        WHERE
                            xt.object_id = $wpdb->comments.comment_post_ID
                    ) OR ";
                    } else {
                        $empty_terms_query = "";
                    }


                    // Quick explanation for the AND
                    // .. MAIN SELECT: selects all object_ids that are not in the array
                    // .. SUBSELECT:   excludes all the object_ids that are part of the array
                    // This is used because of multiple object_ids (posts in more than 1 tag)
                    if ( !empty($exclude_terms_arr[$taxonomy]) ) {
                        $words = implode( ',', $exclude_terms_arr[$taxonomy] );
                        $tax_term_query = " (
                        $empty_terms_query

						$wpdb->comments.comment_post_ID IN (
							SELECT DISTINCT(tr.object_id)
								FROM $wpdb->term_relationships AS tr
				                LEFT JOIN $wpdb->term_taxonomy as tt ON (tr.term_taxonomy_id = tt.term_taxonomy_id AND tt.taxonomy = '$taxonomy')
												WHERE
													tt.term_id NOT IN ($words)
													AND tr.object_id NOT IN (
														SELECT DISTINCT(trs.object_id)
														FROM $wpdb->term_relationships AS trs
				                    LEFT JOIN $wpdb->term_taxonomy as tts ON (trs.term_taxonomy_id = tts.term_taxonomy_id AND tts.taxonomy = '$taxonomy')
														WHERE tts.term_id IN ($words)
													)
										)
									)";
                    }
                    if ( !empty($include_terms_arr[$taxonomy]) ) {
                        $words = implode( ',', $include_terms_arr[$taxonomy] );
                        if ( !empty($tax_term_query) )
                            $tax_term_query .= " AND ";
                        $tax_term_query .= "(
                        $empty_terms_query

                        $wpdb->comments.comment_post_ID IN ( SELECT DISTINCT(tr.object_id)
                            FROM $wpdb->term_relationships AS tr
                            LEFT JOIN $wpdb->term_taxonomy as tt ON (tr.term_taxonomy_id = tt.term_taxonomy_id AND tt.taxonomy = '$taxonomy')
                            WHERE tt.term_id IN ($words)
			          ) )";
                    }

                    if ( !empty($tax_term_query) )
                        $term_query_parts[] = "(" . $tax_term_query . ")";
                }

                if ( !empty($term_query_parts) )
                    $term_query = "AND (" . implode(" ".strtoupper($searchData['taxonomy_logic'])." ", $term_query_parts) . ") ";

                // ---------------------------------------------------------------------


                /*---------------------- Content query --------------------------*/

                $like_part = "";
                $relevance_part = "1";
                $words = $options['set_exactonly'] == 1 ? array($s) : $_s;

                if ($kw_logic == 'or' || $kw_logic == 'and') {
                    $op = strtoupper($kw_logic);
                    if (count($_s) > 0) {
                        $_like = implode("%'$suf_like " . $op . " " . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE $pre_like'%", $words);
                    } else {
                        $_like = $s;
                    }
                    $like_part = "( " . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE $pre_like'%" . $_like . "%'$suf_like )";
                } else {
                    $_like = array();
                    $op = $kw_logic == 'andex' ? 'AND' : 'OR';
                    foreach ($words as $word) {
                        $_like[] = "
                       (" . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE $pre_like'% " . $word . " %'$suf_like
                    OR  " . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE $pre_like'" . $word . " %'$suf_like
                    OR  " . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE $pre_like'% " . $word . "'$suf_like
                    OR  " . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " = '" . $word . "')";
                    }
                    $like_part = "(" . implode(' ' . $op . ' ', $_like) . ")";
                }

                if (count($_s) > 0) {
                    $relevance_part = "(case when
                (" . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE '%" . $_s[0] . "%')
                 then $searchData[contentweight] else 0 end)";
                }

                /*---------------------------------------------------------------*/

                if (isset($options['non_ajax_search']))
                    $limit = 500;
                else
                    $limit = $searchData['maxresults'];

                $querystr = "
      		SELECT 
            $wpdb->comments.comment_ID as id,
            $this->c_blogid as blogid,
            $wpdb->comments.comment_post_ID as post_id,
            $wpdb->comments.user_id as user_id,
            $wpdb->comments.comment_content as content,
            'comment' as content_type,
            $wpdb->comments.comment_date as date,
            $relevance_part as relevance
      		FROM $wpdb->comments
      		WHERE
          ($wpdb->comments.comment_approved=1)
          $term_query
          AND $like_part
      		ORDER BY relevance DESC, $wpdb->comments.comment_ID DESC
      		LIMIT " . $limit;
                //var_dump($querystr);
                $commentsresults = $wpdb->get_results($querystr, OBJECT);
                if (is_array($commentsresults)) {
                    foreach ($commentsresults as $k => $v) {
                        $commentsresults[$k]->link = get_comment_link($v->id);
                        $commentsresults[$k]->author = get_comment_author($v->id);

                        $commentsresults[$k]->title = wd_substr_at_word($commentsresults[$k]->content, 40) . "...";

                    }
                }
            }
            $this->results = $commentsresults;
            return $commentsresults;
        }

    }
}